@@ -8,6 +8,7 @@ use crate::{
 };
 use std::convert::TryInto;
 use std::path::PathBuf;
+use zellij_tile::prelude::Style;
 use zellij_utils::envs::set_session_name;
 use zellij_utils::input::layout::LayoutTemplate;
 use zellij_utils::input::options::Clipboard;
@@ -94,7 +95,7 @@ fn create_new_tab(size: Size) -> Tab {
     let senders = ThreadSenders::default().silently_fail_on_send();
     let max_panes = None;
     let mode_info = ModeInfo::default();
-    let colors = Palette::default();
+    let style = Style::default();
     let draw_pane_frames = true;
     let client_id = 1;
     let session_is_mirrored = true;
@@ -102,17 +103,19 @@ fn create_new_tab(size: Size) -> Tab {
     connected_clients.insert(client_id);
     let connected_clients = Rc::new(RefCell::new(connected_clients));
     let copy_command = None;
+    let character_cell_info = Rc::new(RefCell::new(None));
     let clipboard = Clipboard::default();
     let mut tab = Tab::new(
         index,
         position,
         name,
         size,
+        character_cell_info,
         os_api,
         senders,
         max_panes,
+        style,
         mode_info,
-        colors,
         draw_pane_frames,
         connected_clients,
         session_is_mirrored,
@@ -129,6 +132,16 @@ fn create_new_tab(size: Size) -> Tab {
     tab
 }
 
+fn read_fixture(fixture_name: &str) -> Vec<u8> {
+    let mut path_to_file = std::path::PathBuf::new();
+    path_to_file.push("../src");
+    path_to_file.push("tests");
+    path_to_file.push("fixtures");
+    path_to_file.push(fixture_name);
+    std::fs::read(path_to_file)
+        .unwrap_or_else(|_| panic!("could not read fixture {:?}", &fixture_name))
+}
+
 use crate::panes::grid::Grid;
 use crate::panes::link_handler::LinkHandler;
 use ::insta::assert_snapshot;
@@ -140,6 +153,7 @@ fn take_snapshot(ansi_instructions: &str, rows: usize, columns: usize, palette:
         columns,
         palette,
         Rc::new(RefCell::new(LinkHandler::new())),
+        Rc::new(RefCell::new(None)),
     );
     let mut vte_parser = vte::Parser::new();
     for &byte in ansi_instructions.as_bytes() {
@@ -160,6 +174,7 @@ fn take_snapshot_and_cursor_position(
         columns,
         palette,
         Rc::new(RefCell::new(LinkHandler::new())),
+        Rc::new(RefCell::new(None)),
     );
     let mut vte_parser = vte::Parser::new();
     for &byte in ansi_instructions.as_bytes() {
@@ -828,13 +843,13 @@ fn mark_text_inside_floating_pane() {
     tab.handle_pty_bytes(5, Vec::from("\u{1b}#8".as_bytes()));
     tab.handle_pty_bytes(6, Vec::from("\u{1b}#8".as_bytes()));
     tab.handle_left_click(&Position::new(9, 71), client_id);
-    assert_eq!(
-        tab.selecting_with_mouse, true,
+    assert!(
+        tab.selecting_with_mouse,
         "started selecting with mouse on click"
     );
     tab.handle_mouse_release(&Position::new(8, 50), client_id);
-    assert_eq!(
-        tab.selecting_with_mouse, false,
+    assert!(
+        !tab.selecting_with_mouse,
         "stopped selecting with mouse on release"
     );
     tab.render(&mut output, None);
@@ -1056,3 +1071,27 @@ fn cannot_float_only_embedded_pane() {
     );
     assert_snapshot!(snapshot);
 }
+
+#[test]
+fn replacing_existing_wide_characters() {
+    // this is a real world use case using ncmpcpp with wide characters and scrolling
+    // the reason we don't break it down is that it exposes quite a few edge cases with wide
+    // characters that we should handle properly
+    let size = Size {
+        cols: 238,
+        rows: 48,
+    };
+    let client_id = 1;
+    let mut tab = create_new_tab(size);
+    let mut output = Output::default();
+    let pane_content = read_fixture("ncmpcpp-wide-chars");
+    tab.handle_pty_bytes(1, pane_content);
+    tab.render(&mut output, None);
+    let snapshot = take_snapshot(
+        output.serialize().get(&client_id).unwrap(),
+        size.rows,
+        size.cols,
+        Palette::default(),
+    );
+    assert_snapshot!(snapshot);
+}
