@@ -1,21 +1,22 @@
 use super::{Screen, ScreenInstruction};
+use crate::panes::PaneId;
 use crate::zellij_tile::data::{ModeInfo, Palette};
 use crate::{
-    os_input_output::{AsyncReader, ChildId, Pid, ServerOsApi},
+    os_input_output::{AsyncReader, Pid, ServerOsApi},
     thread_bus::Bus,
-    ClientId, SessionState,
+    ClientId,
 };
 use std::convert::TryInto;
 use std::path::PathBuf;
-use std::sync::{Arc, RwLock};
 use zellij_utils::input::command::TerminalAction;
 use zellij_utils::input::layout::LayoutTemplate;
+use zellij_utils::input::options::Clipboard;
 use zellij_utils::ipc::IpcReceiverWithContext;
-use zellij_utils::pane_size::Size;
+use zellij_utils::pane_size::{Size, SizeInPixels};
 
 use std::os::unix::io::RawFd;
 
-use zellij_utils::ipc::ClientAttributes;
+use zellij_utils::ipc::{ClientAttributes, PixelDimensions};
 use zellij_utils::nix;
 
 use zellij_utils::{
@@ -27,44 +28,48 @@ use zellij_utils::{
 struct FakeInputOutput {}
 
 impl ServerOsApi for FakeInputOutput {
-    fn set_terminal_size_using_fd(&self, fd: RawFd, cols: u16, rows: u16) {
+    fn set_terminal_size_using_fd(&self, _fd: RawFd, _cols: u16, _rows: u16) {
         // noop
     }
-    fn spawn_terminal(&self, _file_to_open: TerminalAction) -> (RawFd, ChildId) {
+    fn spawn_terminal(
+        &self,
+        _file_to_open: TerminalAction,
+        _quit_db: Box<dyn Fn(PaneId) + Send>,
+    ) -> (RawFd, RawFd) {
         unimplemented!()
     }
-    fn read_from_tty_stdout(&self, fd: RawFd, buf: &mut [u8]) -> Result<usize, nix::Error> {
+    fn read_from_tty_stdout(&self, _fd: RawFd, _buf: &mut [u8]) -> Result<usize, nix::Error> {
         unimplemented!()
     }
-    fn async_file_reader(&self, fd: RawFd) -> Box<dyn AsyncReader> {
+    fn async_file_reader(&self, _fd: RawFd) -> Box<dyn AsyncReader> {
         unimplemented!()
     }
-    fn write_to_tty_stdin(&self, fd: RawFd, buf: &[u8]) -> Result<usize, nix::Error> {
+    fn write_to_tty_stdin(&self, _fd: RawFd, _buf: &[u8]) -> Result<usize, nix::Error> {
         unimplemented!()
     }
-    fn tcdrain(&self, fd: RawFd) -> Result<(), nix::Error> {
+    fn tcdrain(&self, _fd: RawFd) -> Result<(), nix::Error> {
         unimplemented!()
     }
-    fn kill(&self, pid: Pid) -> Result<(), nix::Error> {
+    fn kill(&self, _pid: Pid) -> Result<(), nix::Error> {
         unimplemented!()
     }
-    fn force_kill(&self, pid: Pid) -> Result<(), nix::Error> {
+    fn force_kill(&self, _pid: Pid) -> Result<(), nix::Error> {
         unimplemented!()
     }
     fn box_clone(&self) -> Box<dyn ServerOsApi> {
         Box::new((*self).clone())
     }
-    fn send_to_client(&self, client_id: ClientId, msg: ServerToClientMsg) {
+    fn send_to_client(&self, _client_id: ClientId, _msg: ServerToClientMsg) {
         unimplemented!()
     }
     fn new_client(
         &mut self,
-        client_id: ClientId,
-        stream: LocalSocketStream,
+        _client_id: ClientId,
+        _stream: LocalSocketStream,
     ) -> IpcReceiverWithContext<ClientToServerMsg> {
         unimplemented!()
     }
-    fn remove_client(&mut self, client_id: ClientId) {
+    fn remove_client(&mut self, _client_id: ClientId) {
         unimplemented!()
     }
     fn load_palette(&self) -> Palette {
@@ -85,19 +90,29 @@ fn create_new_screen(size: Size) -> Screen {
     };
     let max_panes = None;
     let mode_info = ModeInfo::default();
-    let session_state = Arc::new(RwLock::new(SessionState::new()));
+    let draw_pane_frames = false;
+    let session_is_mirrored = true;
+    let copy_command = None;
+    let copy_clipboard = Clipboard::default();
     Screen::new(
         bus,
         &client_attributes,
         max_panes,
         mode_info,
-        session_state,
-        false, // draw_pane_frames
+        draw_pane_frames,
+        session_is_mirrored,
+        copy_command,
+        copy_clipboard,
     )
 }
 
 fn new_tab(screen: &mut Screen, pid: i32) {
-    screen.apply_layout(LayoutTemplate::default().try_into().unwrap(), vec![pid]);
+    let client_id = 1;
+    screen.new_tab(
+        LayoutTemplate::default().try_into().unwrap(),
+        vec![pid],
+        client_id,
+    );
 }
 
 #[test]
@@ -113,7 +128,7 @@ fn open_new_tab() {
 
     assert_eq!(screen.tabs.len(), 2, "Screen now has two tabs");
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab switched to new tab"
     );
@@ -129,10 +144,10 @@ pub fn switch_to_prev_tab() {
 
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
-    screen.switch_tab_prev();
+    screen.switch_tab_prev(1);
 
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         0,
         "Active tab switched to previous tab"
     );
@@ -148,11 +163,11 @@ pub fn switch_to_next_tab() {
 
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
-    screen.switch_tab_prev();
-    screen.switch_tab_next();
+    screen.switch_tab_prev(1);
+    screen.switch_tab_next(1);
 
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab switched to next tab"
     );
@@ -168,11 +183,11 @@ pub fn close_tab() {
 
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
-    screen.close_tab();
+    screen.close_tab(1);
 
     assert_eq!(screen.tabs.len(), 1, "Only one tab left");
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         0,
         "Active tab switched to previous tab"
     );
@@ -189,27 +204,12 @@ pub fn close_the_middle_tab() {
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
     new_tab(&mut screen, 3);
-    dbg!(screen
-        .tabs
-        .values()
-        .map(|t| (t.index, t.position, t.name.clone(), t.get_pane_ids()))
-        .collect::<Vec<_>>());
-    screen.switch_tab_prev();
-    dbg!(screen
-        .tabs
-        .values()
-        .map(|t| (t.index, t.position, t.name.clone(), t.get_pane_ids()))
-        .collect::<Vec<_>>());
-    screen.close_tab();
-    dbg!(screen
-        .tabs
-        .values()
-        .map(|t| (t.index, t.position, t.name.clone(), t.get_pane_ids()))
-        .collect::<Vec<_>>());
+    screen.switch_tab_prev(1);
+    screen.close_tab(1);
 
     assert_eq!(screen.tabs.len(), 2, "Two tabs left");
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab switched to previous tab"
     );
@@ -226,11 +226,11 @@ fn move_focus_left_at_left_screen_edge_changes_tab() {
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
     new_tab(&mut screen, 3);
-    screen.switch_tab_prev();
-    screen.move_focus_left_or_previous_tab();
+    screen.switch_tab_prev(1);
+    screen.move_focus_left_or_previous_tab(1);
 
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         0,
         "Active tab switched to previous"
     );
@@ -247,11 +247,11 @@ fn move_focus_right_at_right_screen_edge_changes_tab() {
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
     new_tab(&mut screen, 3);
-    screen.switch_tab_prev();
-    screen.move_focus_right_or_next_tab();
+    screen.switch_tab_prev(1);
+    screen.move_focus_right_or_next_tab(1);
 
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         2,
         "Active tab switched to next"
     );
@@ -267,19 +267,19 @@ pub fn toggle_to_previous_tab_simple() {
 
     new_tab(&mut screen, 1);
     new_tab(&mut screen, 2);
-    screen.go_to_tab(1);
-    screen.go_to_tab(2);
+    screen.go_to_tab(1, 1);
+    screen.go_to_tab(2, 1);
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         0,
         "Active tab toggler to previous tab"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab toggler to previous tab"
     );
@@ -298,38 +298,38 @@ pub fn toggle_to_previous_tab_create_tabs_only() {
     new_tab(&mut screen, 3);
 
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1],
         "Tab history is invalid"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab toggler to previous tab"
     );
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(2)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 2],
         "Tab history is invalid"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         2,
         "Active tab toggler to previous tab"
     );
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1],
         "Tab history is invalid"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab toggler to previous tab"
     );
@@ -349,84 +349,196 @@ pub fn toggle_to_previous_tab_delete() {
     new_tab(&mut screen, 4); // 3
 
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1), Some(2)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1, 2],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         3,
         "Active tab toggler to previous tab"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1), Some(3)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1, 3],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         2,
         "Active tab toggler to previous tab"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1), Some(2)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1, 2],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         3,
         "Active tab toggler to previous tab"
     );
 
-    screen.switch_tab_prev();
+    screen.switch_tab_prev(1);
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(1), Some(3)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 1, 3],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         2,
         "Active tab toggler to previous tab"
     );
-    screen.switch_tab_prev();
+    screen.switch_tab_prev(1);
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(3), Some(2)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 3, 2],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab toggler to previous tab"
     );
 
-    screen.close_tab();
+    screen.close_tab(1);
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(3)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 3],
         "Tab history is invalid"
     );
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         1,
         "Active tab toggler to previous tab"
     );
 
-    screen.toggle_tab();
+    screen.toggle_tab(1);
     assert_eq!(
-        screen.get_active_tab().unwrap().position,
+        screen.get_active_tab(1).unwrap().position,
         2,
         "Active tab toggler to previous tab"
     );
     assert_eq!(
-        screen.tab_history,
-        vec![None, Some(0), Some(2)],
+        screen.tab_history.get(&1).unwrap(),
+        &[0, 2],
         "Tab history is invalid"
     );
 }
+
+#[test]
+fn switch_to_tab_with_fullscreen() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let mut screen = create_new_screen(size);
+
+    new_tab(&mut screen, 1);
+    {
+        let active_tab = screen.get_active_tab_mut(1).unwrap();
+        active_tab.new_pane(PaneId::Terminal(2), Some(1));
+        active_tab.toggle_active_pane_fullscreen(1);
+    }
+    new_tab(&mut screen, 2);
+
+    screen.switch_tab_prev(1);
+
+    assert_eq!(
+        screen.get_active_tab(1).unwrap().position,
+        0,
+        "Active tab switched to previous"
+    );
+    assert_eq!(
+        screen
+            .get_active_tab(1)
+            .unwrap()
+            .get_active_pane_id(1)
+            .unwrap(),
+        PaneId::Terminal(2),
+        "Active pane is still the fullscreen pane"
+    );
+}
+
+#[test]
+fn update_screen_pixel_dimensions() {
+    let size = Size {
+        cols: 121,
+        rows: 20,
+    };
+    let mut screen = create_new_screen(size);
+    let initial_pixel_dimensions = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: Some(SizeInPixels {
+            height: 10,
+            width: 5,
+        }),
+        text_area_size: None,
+    });
+    let pixel_dimensions_after_first_update = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: None,
+        text_area_size: Some(SizeInPixels {
+            height: 100,
+            width: 50,
+        }),
+    });
+    let pixel_dimensions_after_second_update = screen.pixel_dimensions;
+    screen.update_pixel_dimensions(PixelDimensions {
+        character_cell_size: None,
+        text_area_size: None,
+    });
+    let pixel_dimensions_after_third_update = screen.pixel_dimensions;
+    assert_eq!(
+        initial_pixel_dimensions,
+        PixelDimensions {
+            character_cell_size: None,
+            text_area_size: None
+        },
+        "Initial pixel dimensions empty"
+    );
+    assert_eq!(
+        pixel_dimensions_after_first_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: None
+        },
+        "character_cell_size updated properly",
+    );
+    assert_eq!(
+        pixel_dimensions_after_second_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: Some(SizeInPixels {
+                height: 100,
+                width: 50,
+            }),
+        },
+        "text_area_size updated properly without overriding character_cell_size",
+    );
+    assert_eq!(
+        pixel_dimensions_after_third_update,
+        PixelDimensions {
+            character_cell_size: Some(SizeInPixels {
+                height: 10,
+                width: 5
+            }),
+            text_area_size: Some(SizeInPixels {
+                height: 100,
+                width: 50,
+            }),
+        },
+        "empty update does not delete existing data",
+    );
+}
